/* Sketch based on:
4-Way Button: Click, Double-Click, Press+Hold, and Press+Long-Hold Test Sketch
By Jeff Saltzman
Oct. 13, 2009
(http://jmsarduino.blogspot.com/2009/10/4-way-button-click-double-click-hold.html)
*/
#include <Servo.h>
Servo myservo;
const int servoPin = 3;

const int buttonPin1 = 2; // analog input pin to use as a digital input
const int stopPin = 4;
boolean stopVal = HIGH;

int counter = 0;

//=================================================

void setup() 
{
  Serial.begin(9600);
// Set button input pin
pinMode(buttonPin1, INPUT);
digitalWrite(buttonPin1, HIGH);
pinMode(stopPin, INPUT);
digitalWrite(stopPin, HIGH);
// Set Servo output pin
myservo.attach(servoPin);

}

void loop()
{

// Get button event and act accordingly
int b = checkButton();
if (b == 1) {
  clickEvent(); //upwards
}
if (b == 2) {
  doubleClickEvent(); //downwards
}
if (b == 3) {
  holdEvent(); //stop;
}
delay(100);
}

//=================================================
// Events to trigger by click and press+hold

void clickEvent() { //down
  myservo.write(0);
  if (counter > 3)
  {
    myservo.write(90);
  }
}

void doubleClickEvent() { //up
  myservo.write(180);
  if (counter > 5)
  {
    myservo.write(90);
  }
}

void holdEvent() { //stop
  myservo.write(90);
}

// Button timing variables
int debounce = 20; // ms debounce period to prevent flickering when pressing or releasing the button
int DCgap = 400; // max ms between clicks for a doubleClickEvent
int holdTime = 200; // ms hold period: how long to wait for holdEevent

// Other button variables
boolean buttonVal = HIGH; // value read from button
boolean buttonLast = HIGH; // buffered value of the button's previous state
boolean DCwaiting = false; // whether we're waiting for a double click (down)
boolean DConUp = false; // whether to register a double click on next release, or whether to wait and click
boolean singleOK = true; // whether it's OK to do a single click
long downTime = -1; // time the button was pressed down
long upTime = -1; // time the button was released
boolean ignoreUp = false; // whether to ignore the button release because the holdEvent was triggered
boolean waitForUp = false; // when held, whether to wait for the up event
boolean holdEventPast = false; // whether or not the hold event happened already

int checkButton()
{
int event = 0;
// Read the state of the button
buttonVal = digitalRead(buttonPin1);
stopVal = digitalRead(stopPin);
  if (buttonVal != buttonLast)
  {
    ++counter;
  }
Serial.print(counter);
Serial.print(buttonVal);
Serial.println(stopVal);

// Button pressed down
if (buttonVal == LOW && buttonLast == HIGH && (millis() - upTime) > debounce) // button pressed, button not pressed before, time since release larger than debounce
{
downTime = millis(); // time of press = time now
ignoreUp = false; // ignore button release
waitForUp = false; // do not wait for up event
singleOK = true; // okay to do a single clickEvent
holdEventPast = false; // holdEvent did not happen yet
if ((millis()-upTime) < DCgap && DConUp == false && DCwaiting == true) // additionally: wait for a doubleClickEvent, if time since first click smaller than DCgap
{
  DConUp = true; // do not register double click on next release, do wait and click
}
else // if time since first click larger than DC gap
{
  DConUp = false; // do register a double click on next release
}
DCwaiting = false; // do not wait for double click
}
// Button released
else if (buttonVal == HIGH && buttonLast == LOW && (millis() - downTime) > debounce) // button released, button pressed before, time since release larger than debounce
{ 
if (not ignoreUp) // if ignoreUp == false, if button release was ignored because click+hold was triggered
{
upTime = millis(); // time of release = time now
if (DConUp == false) // do not register a doubleClick on next release, do wait and click
{
  DCwaiting = true; // wait for a double click
}
else
{
event = 2; // doubleClickEvent happens!
DConUp = false; // do not register a doubleClick on next release, do wait and click
DCwaiting = false; // no not wait for doubleClick
singleOK = false; // not ok to do a singleClick
}
}
}
// Test for normal click event: DCgap expired
if ( buttonVal == HIGH && (millis()-upTime) >= DCgap && DCwaiting == true && DConUp == false && singleOK == true) // button not pressed and time since press higher than gap, waiting for double click, ok to do single click
{
event = 1; // clickEvent happens!
DCwaiting = false; // do not wait for double click
}
// Test for hold
if (buttonVal == LOW && (millis() - downTime) >= holdTime) // button pressed, time since press down larger than hold Time
{
// Trigger "normal" hold
if (not holdEventPast) // if holdEventPast = false, if holdEvent did not happen yet
{
event = 3; // holdEvent happens!
counter = 0;
waitForUp = true; // wait for up event
ignoreUp = true; // ignore button release since holdEvent was triggered
DConUp = false; // do not register a double click on next release, wait and click
DCwaiting = false; // do not wait for double click
//downTime = millis();
holdEventPast = true; // hold event did happen already
}
}
buttonLast = buttonVal; // make button state as previous state

if ((counter == 5) && (stopVal == HIGH))
{
  event = 3;
}
return event;
}
