/* Sketch based on:
4-Way Button: Click, Double-Click, Press+Hold, and Press+Long-Hold Test Sketch
By Jeff Saltzman
Oct. 13, 2009
(http://jmsarduino.blogspot.com/2009/10/4-way-button-click-double-click-hold.html)
*/
#include <Servo.h>
Servo myservo;
const int servoPin = 3; // servo

const int buttonPin = 2; // infrared switch
const int stopPin = 4; // endstop switch 
boolean stopVal = HIGH;

int counter = 0;
int counterLast = 0;
boolean dir = 1; // initial direction after turning it on is upwards
boolean dirStop = 1;
int event = 0;

//=================================================

void setup() 
{
  Serial.begin(9600);
// Set button input pin
pinMode(buttonPin, INPUT);
digitalWrite(buttonPin, HIGH);
// Set endstop input pin
pinMode(stopPin, INPUT);
digitalWrite(stopPin, HIGH);
// Set Servo output pin
myservo.attach(servoPin);

}

//check, if movement or stop
void loop()
{
// Get button event and act accordingly
int b = checkButton();
if (b == 1) {
    myservo.write(0); //move upwards
}
if (b == 2) {
    myservo.write(180); //move downwards
}
if (b == 3) {
    myservo.write(90);  //do not move
}
delay(200);
}

// Button timing variables
int debounce = 20; // ms debounce period to prevent flickering when pressing or releasing the button
int holdTime = 200; // ms hold period: how long to wait for holdEvent

// Other button variables
boolean buttonVal = HIGH; // value read from button
boolean buttonLast = HIGH; // buffered value of the button's previous state
//boolean singleOK = true; // whether it's OK to do a single click
long downTime = -1; // time the button was pressed down
long upTime = -1; // time the button was released
boolean ignoreUp = false; // whether to ignore the button release because the holdEvent was triggered
boolean waitForUp = false; // when held, whether to wait for the up event
boolean holdEventPast = false; // whether or not the hold event happened already
boolean stopEventPast = false; // whether or not the stop button was pressed already

int checkButton()
{
//  int event = 0;
  // Read the state of the button
buttonVal = digitalRead(buttonPin);
stopVal = digitalRead(stopPin);
  if (buttonVal != buttonLast) // if the button has changed since last read, increment counter
  {
    ++counter;
  }

// Moment of button pressing down
if (buttonVal == LOW && buttonLast == HIGH && (millis() - upTime) > debounce) // button pressed, button not pressed before, time since release larger than debounce
{
downTime = millis(); // time of press = time now
holdEventPast = false; // holdEvent did not happen yet
}
// Test for hold
if (buttonVal == LOW && (millis() - downTime) >= holdTime) // button pressed, look if time since press down larger than hold Time
{
// if YES: Trigger "normal" hold
if (not holdEventPast) // if holdEventPast = false, if holdEvent did not happen yet, if hold is not already active
{
counter = 0; //holdEvent happens!
downTime = millis();
holdEventPast = true; // hold event did happen already
}
}
// change direction once after click
if (counter > 1 && counter%2 == 1 && counter != counterLast )
{
 dir = !dir;
}

// if end-stop pressed, stop and move to opposite direction after next click
if (stopVal == HIGH)
{
  if (not stopEventPast) // if the end stop was not pressed already
  {
    counter = 0;
    stopEventPast = true;
    dirStop = dir;
  }
  
  else if (dirStop == dir && counter > 2)
  {
    counter = 0;
  }
  else if (dirStop != dir)
  {
    stopEventPast = false;
  }
  }

//selection of movement sent to servo:
if (counter > 2 && dir == 0) //not held down and movement up
{
  event = 1;
}
else if (counter > 2 && dir == 1) //not held down and movement down
{
  event = 2;
}
else if (counter < 3) //held down
{
  event = 3;
}

buttonLast = buttonVal; // make button state as previous state
counterLast = counter; // make counter as previous counter

Serial.print(counter);
//Serial.print(buttonVal);
//Serial.print(stopVal);
Serial.print(event);
Serial.print(stopEventPast);
Serial.println(dir);
return event;
}
